
const fs = require('fs');
const { resolve } = require("path");
const packageJSON = require("../package.json");
const { mkDir } = require("./tools");



/**
 * update.flag  每次更新开始前将其置为true,更新成功后删除,防止更新一半的情况
 * 此处需要返回[update.flag,index,next,depend,init]的最新内容
 */
const getBootFiles = (userDataPath, jsInterceptID, paths) => {
    paths = JSON.parse(paths);
    console.log("getBootFiles ", paths);
    console.time("getBootFiles ");
    const arr = paths.map(item => {
        if (item.endsWith("update.flag")) return getUpdateFlag(userDataPath, item);
        return getBootFile(userDataPath, item);
    });
    console.timeEnd("getBootFiles ");
    return [jsInterceptID, true, JSON.stringify(arr)];
}

const getBootFile = (userDataPath, path) => {
    const absolutePath = resolve(userDataPath, "static", path);
    if (fs.existsSync(absolutePath)) {
        const content = fs.readFileSync(absolutePath, "utf-8")
        return new Buffer(content).toString('base64');
    };

    return "";
}

const getUpdateFlag = (userDataPath, path) => {
    const updatePath = resolve(userDataPath, "static", path);
    if (fs.existsSync(updatePath)) return fs.readFileSync(updatePath, "utf-8")
    return "false";
}
/**
 * 获取app版本 
 * exe版暂时不考虑exe更新
 * isOK, _pkgName, version, justUpdate
 */
const getAppVersion = (jsInterceptID) => {
    const _pkgName = packageJSON.appId;
    const version = packageJSON.version;
    const justUpdate = 0;
    return [jsInterceptID, true, _pkgName, version, justUpdate];
}

/**
 * 保存文件
 */
const saveFile = (userDataPath, path, content, jsInterceptID) => {
    console.log("!!!!!!!!!!saveFile = ", path);
    if (path.endsWith(".depend")) path = path.replace(".", "");
    path = resolve(userDataPath, "static", path);
    mkDir(path);
    fs.writeFileSync(path, content, "utf-8");
    return [jsInterceptID];
}

exports.getBootFiles = getBootFiles;
exports.getAppVersion = getAppVersion;
exports.saveFile = saveFile;