const sqlite3 = require('sqlite3');
const fs = require("fs");
const resolve = require('path').resolve;
const { mkDir } = require("../tools");

const CREATE_TABLE_SQL = "CREATE TABLE IF NOT EXISTS %@ ( id TEXT NOT NULL, json TEXT NOT NULL, PRIMARY KEY(id))";
const UPDATE_ITEM_SQL = "REPLACE INTO %@ (id, json) values (?,?)";
const QUERY_ITEM_SQL = "SELECT json from %@ where id = ? Limit 1";
const DELETE_ITEM_SQL = "DELETE from %@ where id = ?";
const SELECT_ALL_SQL = "SELECT * from %@";
const CLEAR_ALL_SQL = "DELETE from %@";

class PiV8Db {
    constructor(appPath) {
        const path = resolve(appPath, "databases");
        if (!fs.existsSync(path)) {
            mkDir(path);
            fs.mkdirSync(path)
        };
        this.db = new sqlite3.Database(`${path}/piv8.db`);
    }
    create(tabName, success, fail, complete) {
        const createSql = CREATE_TABLE_SQL.replace("%@", tabName);
        this.db.run(createSql, function (err) {
            if (err) {
                fail && fail(err);
            } else {
                success && success();
            }
            complete && complete();
        });
    }

    write(tabName, key, data, success, fail, complete) {
        const updateSql = UPDATE_ITEM_SQL.replace("%@", tabName);
        this.db.run(updateSql, [key, data], function (err) {
            if (err) {
                fail && fail(err);
            } else {
                success && success();
            }
            complete && complete();
        });
    }

    read(tabName, key, success, fail, complete) {
        const querySql = QUERY_ITEM_SQL.replace("%@", tabName);
        this.db.get(querySql, [key], function (err, content) {
            if (err) {
                fail && fail(err);
            } else {
                success && success(new DataHandler(content && content.json));
            }
            complete && complete();
        });
    }

    remove(tabName, key, success, fail, complete) {
        const removeSql = DELETE_ITEM_SQL.replace("%@", tabName);
        this.db.run(removeSql, [key], function (err, content) {
            if (err) {
                fail && fail(err);
            } else {
                success && success();
            }
            complete && complete();
        });
    }

    iterate(tabName, success, fail, complete) {
        const quertyAllsql = SELECT_ALL_SQL.replace("%@", tabName);
        this.db.all(quertyAllsql, function (err, content) {
            if (err) {
                fail && fail(err);
            } else {
                let obj = {};
                content.forEach(item => obj[item["id"]] = item["json"]);
                success && success(JSON.stringify(obj));
            }
            complete && complete();
        });

    }

    delete(tabName, success, fail, complete) {
        const deleteTabSql = CLEAR_ALL_SQL.replace("%@", tabName);
        this.db.run(deleteTabSql, function (err, content) {
            if (err) {
                fail && fail(err);
            } else {
                success && success();
            }
            complete && complete();
        });
    }
}

class DataHandler {
    constructor(data) {
        this.data = data;
    }
    getContent() {
        return this.data;
    }
}

exports.PiV8Db = PiV8Db;