
const { ipcMain } = require('electron');
console.log("我是rpc");

let sendChannel = "pi-rpc-main";
let receiveChannel = "pi-rpc-render";
let _sender;


const setSender = (sender) => {
    _sender = sender;
}
/**
 * 特殊的消息开头，代表这是一个RPC调用
 */
const RPC_CALL_START = '$WEBVIEW_RPC_CALL: ';

/**
 * 特殊的消息开头，代表这是一个RPC回应
 */
const RPC_REPLY_START = '$WEBVIEW_RPC_REPLY: ';

/**
 * 特殊的消息格式，代表参数是一个函数
 */
const RPC_CALLBACK_PARAM = '$WEBVIEW_RPC_FUNCTION_PARAM: ';

/**
 * rpc的当前可用的id 和 RPC映射表
 */
let rpcCurrentID = 0;
const rpcMap = new Map();



const rpcCall = (sender, moduleName, methodName, params, callback) => {
    sender = sender || _sender;
    const funcs = [];
    params = params || [];
    params = params.map(v => {
        if (v === undefined) {
            v = null;
        } else if (v instanceof Function) {
            const id = funcs.length;
            funcs.push(v);
            v = RPC_CALLBACK_PARAM + id;
        }

        return v;
    });

    const sign = {
        moduleName,
        methodName,
        params
    };
    if (callback) {
        sign.resultCallbackID = funcs.length;
        funcs.push(callback);
    }

    if (funcs.length > 0) {
        sign.rpcID = ++rpcCurrentID;
        rpcMap.set(sign.rpcID, funcs);
    }
    const message = RPC_CALL_START + JSON.stringify(sign);
    sender.send(sendChannel, message);
};





/**
 * 收到对方RPC之后的处理
 */
const handleRpcCall = (sender, { moduleName, methodName, params, rpcID, resultCallbackID }) => {
    let func, result;
    pi_modules.commonjs.exports.require([moduleName], {}, (mods, fm) => {
        const mod = mods[0];
        func = mod[methodName];
        if (!func) {
            result = {
                error: `throw error, can\'t find module '${moduleName}', function = '${methodName}`
            };
        }
        if (func) {
            /**
             * 将参数的回调函数恢复回来
             */
            params = params.map(v => {
                if (typeof v === 'string' && v.startsWith(RPC_CALLBACK_PARAM)) {
                    const id = JSON.parse(v.slice(RPC_CALLBACK_PARAM.length));

                    return (...args) => {
                        const sign = {
                            args: args,
                            rpcID: rpcID,
                            callbackID: id
                        };

                        const message = RPC_REPLY_START + JSON.stringify(sign);
                        sender.send(sendChannel, message);
                    };
                }

                return v;
            });

            try {
                const callback = (...args) => {
                    if (resultCallbackID === undefined) return;
                    const sign = {
                        args: args,
                        rpcID: rpcID,
                        callbackID: resultCallbackID
                    };
                    const message = RPC_REPLY_START + JSON.stringify(sign);
                    sender.send(sendChannel, message);
                };
                result = func(...params, callback);
            } catch (e) {
                func = undefined;
                result = {
                    error: 'call throw error'
                };
            }
        }
        // 异常情况时，func为undefined，这时必须让对方的rpc释放掉
        if (!func && resultCallbackID !== undefined) {
            const sign = {
                args: [[result]],
                rpcID,
                callbackID: resultCallbackID
            };

            const message = RPC_REPLY_START + JSON.stringify(sign);
            sender.send(sendChannel, message);
        }

    }, (result) => {
        result = {
            error: `throw error, load fail ${moduleName}`
        };
        const sign = {
            args: [[result]],
            rpcID,
            callbackID: resultCallbackID
        };

        const message = RPC_REPLY_START + JSON.stringify(sign);
        sender.send(sendChannel, message);
    }, () => { });


};


/**
 * 收到对方RPC之后的处理
 */
const handleRpcCall1 = (sender, { methodName, params, rpcID, resultCallbackID }) => {
    let func, result;
    func = pi_remote[methodName];
    if (!func) {
        result = {
            error: `throw error, can\'t find module '${moduleName}', function = '${methodName}`
        };
    }
    if (func) {
        /**
         * 将参数的回调函数恢复回来
         */
        params = params.map(v => {
            if (typeof v === 'string' && v.startsWith(RPC_CALLBACK_PARAM)) {
                const id = JSON.parse(v.slice(RPC_CALLBACK_PARAM.length));

                return (...args) => {
                    const sign = {
                        args: args,
                        rpcID: rpcID,
                        callbackID: id
                    };

                    const message = RPC_REPLY_START + JSON.stringify(sign);
                    sender.send(sendChannel, message);
                };
            }

            return v;
        });

        try {
            const callback = (...args) => {
                if (resultCallbackID === undefined) return;
                const sign = {
                    args: args,
                    rpcID: rpcID,
                    callbackID: resultCallbackID
                };
                const message = RPC_REPLY_START + JSON.stringify(sign);
                sender.send(sendChannel, message);
            };
            result = func(...params, callback);
        } catch (e) {
            func = undefined;
            result = {
                error: 'call throw error'
            };
        }
    }
    // 异常情况时，func为undefined，这时必须让对方的rpc释放掉
    if (!func && resultCallbackID !== undefined) {
        const sign = {
            args: [[result]],
            rpcID,
            callbackID: resultCallbackID
        };

        const message = RPC_REPLY_START + JSON.stringify(sign);
        sender.send(sendChannel, message);
    }
};

/**
 * 收到对方RPC回应之后的处理
 */
const handleRpcReply = ({ rpcID, callbackID, args }) => {
    const funcs = rpcMap.get(rpcID);
    const f = funcs && funcs[callbackID];
    if (f) {
        f(...args);
    }
    rpcMap.delete(rpcID);
};

/**
 * 设置ipc监听
 */
ipcMain.on(receiveChannel, (event, message) => {
    // 收到对方的rpc调用请求，处理
    if (message.startsWith(RPC_CALL_START)) {
        message = message.slice(RPC_CALL_START.length);
        const data = JSON.parse(message);
        if (!data.moduleName) return handleRpcCall1(event.sender, data); // 调用远程本地方法
        return handleRpcCall(event.sender, data);
    }

    // 收到对方的rpc回应，处理
    if (message.startsWith(RPC_REPLY_START)) {
        message = message.slice(RPC_REPLY_START.length);
        const data = JSON.parse(message);

        return handleRpcReply(data);
    }
});

exports.rpcCall = rpcCall;
exports.setSender = setSender;
