const fs = require('fs');
const { dirname, resolve } = require("path");
const fetch = require('node-fetch');
const packageJSON = require("../package.json");
const config = require("../config.json");

// 递归创建目录
const mkDir = (path) => {
    const dir = dirname(path);
    if (fs.existsSync(dir)) return true;
    if (mkDir(dir)) {
        fs.mkdirSync(dir);
        return true;
    }
}

// 加载本地vm.js
const loadVmJS = async (userDataPath) => {
    const arr = packageJSON.version.split(".").slice(0, 2);
    const fileName = `${arr.join("_")}.json`;
    let hasUpdated = false;
    if (fs.existsSync(resolve(userDataPath, "static", `${config.js_path}/${fileName}`))) hasUpdated = true;
    let dirPath;
    if (hasUpdated) {
        dirPath = resolve(userDataPath, "static");
    } else {
        dirPath = resolve(__dirname, "../assets");
    }
    const localContent = fs.readFileSync(resolve(dirPath, `${config.js_path}/${fileName}`), "utf-8");
    const localVmPath = JSON.parse(localContent).vm;
    let requireVmPath;
    try {
        const res = await fetch(`${config.remote_path}/${config.js_path}/${fileName}`);
        if (!res.ok) throw new Error(res.statusText);;
        const json = await res.json();
        const remoteVmPath = json.vm
        if (remoteVmPath === localVmPath) {
            requireVmPath = resolve(dirPath, localVmPath);
        } else {
            console.log("vm有更新");
            const res = await fetch(`${config.remote_path}/${remoteVmPath}`);
            if (!res.ok) throw new Error(res.statusText);;
            const rPath = resolve(userDataPath, "static", remoteVmPath);
            mkDir(rPath);
            const text = await res.text();
            fs.writeFileSync(rPath, text, "utf-8");
            const lPath = resolve(userDataPath, "static", `${config.js_path}/${fileName}`);
            mkDir(lPath);
            fs.writeFileSync(lPath, JSON.stringify(json), "utf-8");
            if (hasUpdated) {  // 更新过  删除上次更新的vm
                fs.unlink(resolve(userDataPath, "static", localVmPath), (err) => {
                    if (err) {
                        console.log("删除失败");
                    } else {
                        console.log("删除成功");
                    }
                });
            }
            requireVmPath = resolve(userDataPath, "static", remoteVmPath);
        }

    } catch (err) {    // 失败   默认读取本地最新版本
        console.error("!! ", err);
        requireVmPath = resolve(dirPath, localVmPath);
    }
    require(requireVmPath);
}




exports.mkDir = mkDir;
exports.loadVmJS = loadVmJS;